

# KalmanFilter
MEASUREMENT_SPACE_DIM = 4
POS_WEIGHT_DEFAULT = 1/20
VEL_WEIGHT_DEFAULT = 1/160
POS_WEIGHT_FACTOR_INIT = 2
VEL_WEIGHT_FACTOR_INIT = 10
AR_PROJ_STD_DEFAULT = 1e-1
AR_POS_STD_DEFAULT = 1e-2
AR_VEL_STD_DEFAULT = 1e-5
STATE_MEAN_HEIGHT_IDX = 3
"""
GATING_THRESHOLD: 
The x value covering 95% of the area 
of a 4th degree chisquare distirbution
"""
GATING_THRESHOLD = 9.487729036781154
